indexing
  description: "A basic savings account"
  author: "Gunnar Gotshalks"
  date: "$2004 January 30$"
  revision: "$1.0.0$"

class SAVINGS_ACCOUNT

-- A basic savings account that can be opened with an initial balance
-- and then deposits and withdrawals can be made.

create open

--========================
feature {NONE} -- creation

open(initial_deposit:REAL) is
    -- Open a savings account with initial balance of 'initial_deposit'
  require
    positive_deposit: initial_deposit >= 0
  do
    balance := initial_deposit
  ensure
  	open_balance_ok: balance = initial_deposit
  end

--========================
feature -- Access

  balance: REAL    -- Running total of deposits & withdrawals beginning
                   -- with initial_depost

--========================
feature -- Element change

deposit (amount: REAL) is
    -- Add the amount to the account balance
  require
  	positive_amount: amount> 0
  do
  	balance := balance + amount
  ensure
    proper_balance: balance = old balance + amount
  end


withdraw (amount: REAL) is
    -- Subtract the amount from the account balance
  require
    positive_amount: amount > 0
    amount <= balance
  do
  	balance := balance - amount
  ensure
  	proper_balance: balance = old balance - amount
  end

invariant
  balance_not_negative: balance >= 0

end -- class SAVINGS_ACCOUNT
