indexing
  description: "Tests for SAVINGS_ACCOUNT"
  author: "Gunnar Gotshalks"
  date: "$2004 January 30$"
  revision: "$1.0.0$"

class TEST_SAVINGS_ACCOUNT
  inherit ES_TEST

create make

--========================
feature {NONE} -- Initialization
make is
    -- Run tests
  do
    all_violation_cases
    all_boolean_cases
    show_browser
    run_espec
  end

--========================
feature -- Violation

all_violation_cases is
  -- Gather all violation cases together to simplify turning off while
  -- working on the boolean cases
  do
    add_violation_case (agent create_negative_amount)
    add_violation_case_with_tag ("positive_deposit", agent create_negative_amount)
    add_violation_case (agent deposit_zero_amount)
    add_violation_case (agent withdraw_zero_amount)
    add_violation_case (agent withdraw_gt_balance)
 end

create_negative_amount is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
   do
    comment("Precondition create -1, negative at boundary")
    initial_deposit := -1
    create the_account.open(initial_deposit)
  end

deposit_zero_amount is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
    the_amount : REAL
   do
    comment("Precondition deposit 0, at boundary")
    initial_deposit := 500  ; the_amount := 0
    create the_account.open(initial_deposit)
    the_account.deposit(the_amount)
  end

withdraw_zero_amount is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
    the_amount : REAL
   do
    comment("Precondition withdraw 0, at boundary")
    initial_deposit := 500  ; the_amount := 0
    create the_account.open(initial_deposit)
    the_account.withdraw(the_amount)
  end

withdraw_gt_balance is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
    the_amount : REAL
   do
    comment("Precondition withdraw balance+1 at boundary")
    initial_deposit := 500  ; the_amount := 501
    create the_account.open(initial_deposit)
    the_account.withdraw(the_amount)
  end

--========================
feature -- Boolean

all_boolean_cases is
  -- Gather all boolean cases as a unit.
  do
  	add_boolean_case(agent withdraw_deposit)
    add_boolean_case (agent open_account_minimum)
    add_boolean_case (agent open_account_gt_minimum)
    add_boolean_case (agent deposit_minimum)
    add_boolean_case (agent deposit_gt_minimum)
    add_boolean_case (agent withdraw_minimum)
    add_boolean_case (agent withdraw_partial_balance)
    add_boolean_case (agent withdraw_maximum)
 end

withdraw_deposit: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Get original balance on withdrawing a deposit?")
    initial_deposit := 5  ;  amount:= 3
    create the_account.open(initial_deposit)
    the_account.deposit(amount)
    the_account.withdraw(amount)
    Result := initial_deposit = the_account.balance
  end

open_account_minimum: BOOLEAN is
  -- Is a balance of 0 dollars created correctly
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
  do
    comment("Create account with 0 dollar balance, boundary amount")
    initial_deposit := 0
    create the_account.open(initial_deposit)
    Result := the_account.balance = initial_deposit
  end

open_account_gt_minimum: BOOLEAN is
  -- Is a balance far from the boundary created correctly
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
  do
    comment("Create account with 500 dollar balance, far from boundary amount")
    initial_deposit := 500
    create the_account.open(initial_deposit)
    Result := the_account.balance = initial_deposit
  end

deposit_minimum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Deposit 1, the boundary amount")
    initial_deposit := 500 ; amount := 1
    create the_account.open(initial_deposit)
    the_account.deposit(amount)
    Result := the_account.balance = initial_deposit + amount
  end

deposit_gt_minimum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Deposit far from the boundary amount")
    initial_deposit := 500 ; amount := 100
    create the_account.open(initial_deposit)
    the_account.deposit(amount)
    Result := the_account.balance = initial_deposit + amount
  end

withdraw_minimum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Withdraw 1, the boundary amount")
    initial_deposit := 500 ; amount := 1
    create the_account.open(initial_deposit)
    the_account.withdraw(amount)
    Result := the_account.balance = initial_deposit - amount
  end

withdraw_partial_balance: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Withdraw a partial amount of the balance, non-boundary")
    initial_deposit := 500 ; amount := 100
    create the_account.open(initial_deposit)
    the_account.withdraw(amount)
    Result := the_account.balance = initial_deposit - amount
  end

withdraw_maximum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Withdraw the maximum amount, boundary")
    initial_deposit := 500 ; amount := 100
    create the_account.open(initial_deposit)
    the_account.withdraw(amount)
    Result := the_account.balance = initial_deposit - amount
  end

end -- class TEST_SAVINGS_ACCOUNT
