indexing
  description: "A basic savings account"
  author: "Gunnar Gotshalks"
  date: "$2004 January 31$"
  revision: "$1.0.0$"

class SAVINGS_ACCOUNT 

-- A savings account that can be created at one time but no transactions
-- can be made until the account is opened with an initial balance.  At which
-- point deposits and withdrawals can be made.  Eventually the account can
-- be closed.  After closing no further transactions can be made but the
-- balance can be inspected.

create make

--========================
feature {NONE} -- creation

make is
    -- create an unopened savings account.  Transactions cannot be done until
    -- the account is opened.
  require true
  do
    status := acct_not_open
  ensure
    status = acct_not_open
  end

--========================
feature -- Access
  
  balance : REAL    -- Running total of deposits & withdrawals beginning
                    -- with initial_depost
  status  : REAL    -- State of the account is one of the following.
  acct_not_open, acct_open, acct_closed : INTEGER is unique

--========================
feature -- Element change

open(initial_deposit:REAL) is
    -- Open a savings account with initial balance of 'initial_deposit'
    -- After opening it transactions on the account can be done.
  require
    status = acct_not_open
    initial_deposit >= 0
  do
    balance := initial_deposit ; status := acct_open
  ensure
    balance = initial_deposit and status = acct_open
  end

close is
  -- Close an opened savings account.  No further transactions can be done
  -- but the balance can be inspected.
  require  status = acct_open
  do
    status := acct_closed
  ensure
    status = acct_closed
  end
    
deposit (amount: REAL) is
    -- Add the amount to the account balance
  require
    status = acct_open
    amount_positive: amount> 0
  do
    balance := balance + amount
  ensure
    balance = old balance + amount
    status = old status
  end

withdraw (amount: REAL) is
    -- Subtract the amount from the account balance
  require
    status = acct_open
    amount > 0
    amount <= balance
  do
    balance := balance - amount
  ensure
    balance = old balance - amount
    status = old status
  end

invariant
  balance >= 0

end -- class SAVINGS_ACCOUNT
