indexing
  description: "Tests for SAVINGS_ACCOUNT in bank"
  author: "Gunnar Gotshalks"
  date: "$2004 January 30$"
  revision: "$1.0.0$"

class TEST_SAVINGS_ACCOUNT
  inherit ES_TEST

create  make

--========================
feature {NONE} -- Initialization
make is
    -- Run tests
  do
    create bc
    all_violation_cases
    all_boolean_cases
  end

bc : BANK_CONSTANTS

--========================
feature -- Violation

all_violation_cases is
  -- Gather all violation cases together to simplify turning off while
  -- working on the boolean cases
  do
    add_violation_case (agent open_invalid_status)
    add_violation_case (agent open_negative_amount)
    add_violation_case (agent deposit_invalid_status)
    add_violation_case (agent deposit_zero_amount)
    add_violation_case (agent withdraw_invalid_status)
    add_violation_case (agent withdraw_zero_amount)
    add_violation_case (agent withdraw_gt_balance)
    add_violation_case (agent close_invalid_status)
 end

open_negative_amount is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
   do
    comment("Precondition create -1, negative at boundary")
    initial_deposit := -1
    create the_account.make
    the_account.open(initial_deposit)
  end

open_invalid_status is
  -- Invalid open for second open and after close.
  -- Test for after close id done by a correct status
  -- change after close.
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
  do
    comment("Precondition second open should not be possible")
    initial_deposit := 500
    create the_account.make
    the_account.open(initial_deposit)
    the_account.open(initial_deposit)
  end

deposit_zero_amount is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
    amount : REAL
   do
    comment("Precondition deposit 0, at boundary")
    initial_deposit := 500  ; amount := 0
    create the_account.make
    the_account.open(initial_deposit)
    the_account.deposit(amount)
  end

deposit_invalid_status is
  local
    the_account : SAVINGS_ACCOUNT
    amount : REAL
  do
    comment("Precondition for open -- account must be open")
    amount := 500
    create the_account.make
    the_account.deposit(amount)
  end

withdraw_zero_amount is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
    amount : REAL
   do
    comment("Precondition withdraw 0, at boundary")
    initial_deposit := 500  ; amount := 0
    create the_account.make
    the_account.open(initial_deposit)
    the_account.withdraw(amount)
  end

withdraw_gt_balance is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit : REAL
    amount : REAL
   do
    comment("Precondition withdraw balance+1 at boundary")
    initial_deposit := 500  ; amount := 501
    create the_account.make
    the_account.open(initial_deposit)
    the_account.withdraw(amount)
  end

withdraw_invalid_status is
  -- The only way to test this is if the status precondition is first
  -- the other two preconditions imply status = open.
  local
    the_account : SAVINGS_ACCOUNT
    amount : REAL
  do
    comment("Precondition for withdraw -- account must be open")
    amount := 500
    create the_account.make
    the_account.withdraw(amount)
  end

close_invalid_status is
  local
    the_account : SAVINGS_ACCOUNT
  do
    comment("Precondition for close -- account must be open")
    create the_account.make
    the_account.close
  end

--========================
feature -- Account

all_boolean_cases is
  -- Gather all boolean cases as a unit.
  do
    add_boolean_case (agent withdraw_deposit)
    add_boolean_case (agent open_account_minimum)
    add_boolean_case (agent open_account_gt_minimum)
    add_boolean_case (agent deposit_minimum)
    add_boolean_case (agent deposit_gt_minimum)
    add_boolean_case (agent withdraw_minimum)
    add_boolean_case (agent withdraw_partial_balance)
    add_boolean_case (agent withdraw_maximum)
    add_boolean_case (agent transaction_counts_correct)
    add_boolean_case (agent transactions_data_correct)
 end

withdraw_deposit: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Get original balance on withdrawing a deposit?")
    initial_deposit := 5  ;  amount:= 3
    create the_account.make
    the_account.open(initial_deposit)
    the_account.deposit(amount)
    the_account.withdraw(amount)
    Result := initial_deposit = the_account.balance
  end

open_account_minimum: BOOLEAN is
  -- Is a balance of 0 dollars created correctly
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
  do
    comment("Create account with 0 dollar balance, boundary amount")
    initial_deposit := 0
    create the_account.make
    the_account.open(initial_deposit)
    Result := the_account.balance = initial_deposit
  end

open_account_gt_minimum: BOOLEAN is
  -- Is a balance far from the boundary created correctly
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
  do
    comment("Create account with 500 dollar balance, far from boundary amount")
    initial_deposit := 500
    create the_account.make
    the_account.open(initial_deposit)
    Result := the_account.balance = initial_deposit
  end

deposit_minimum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Deposit 1, the boundary amount")
    initial_deposit := 500 ; amount := 1
    create the_account.make
    the_account.open(initial_deposit)
    the_account.deposit(amount)
    Result := the_account.balance = initial_deposit + amount
  end

deposit_gt_minimum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Deposit far from the boundary amount")
    initial_deposit := 500 ; amount := 100
    create the_account.make
    the_account.open(initial_deposit)
    the_account.deposit(amount)
    Result := the_account.balance = initial_deposit + amount
  end

withdraw_minimum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Withdraw 1, the boundary amount")
    initial_deposit := 500 ; amount := 1
    create the_account.make
    the_account.open(initial_deposit)
    the_account.withdraw(amount)
    Result := the_account.balance = initial_deposit - amount
  end

withdraw_partial_balance: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Withdraw a partial amount of the balance, non-boundary")
    initial_deposit := 500 ; amount := 100
    create the_account.make
    the_account.open(initial_deposit)
    the_account.withdraw(amount)
    Result := the_account.balance = initial_deposit - amount
  end

withdraw_maximum: BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
    comment("Withdraw the maximum amount, boundary")
    initial_deposit := 500 ; amount := 100
    create the_account.make
    the_account.open(initial_deposit)
    the_account.withdraw(amount)
    Result := the_account.balance = initial_deposit - amount
  end

transaction_counts_correct : BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
  do
  	comment("Make count is 1")
    initial_deposit := 500 ; amount := 100
    create the_account.make
    Result := the_account.transaction_list.count = 1
  	if Result then comment("Open count is 2")
  	  the_account.open(initial_deposit)
      Result := the_account.transaction_list.count = 2
  	end
  	if Result then comment("Deposit count is 3")
      the_account.deposit(amount)
      Result := the_account.transaction_list.count = 3
    end
  	if Result then comment("Withdraw count is 4")
      the_account.withdraw(amount)
      Result := the_account.transaction_list.count = 4
    end
  	if Result then comment("Closed count is 5")
      the_account.close
      Result := the_account.transaction_list.count = 5
    end
  	if Result then comment("Transaction counts increased correctly") end
  end

transactions_data_correct : BOOLEAN is
  local
    the_account : SAVINGS_ACCOUNT
    initial_deposit: REAL
    amount : REAL
    transaction : TRANSACTION
  do
  	comment("Transaction data correct for make")
    initial_deposit := 500 ; amount := 100
    create the_account.make
    transaction := the_account.transaction_list.last
    Result := transaction.account = the_account and transaction.type = bc.trans_create
  	if Result then comment("Transaction data correct for open")
  	  the_account.open(initial_deposit)
      transaction := the_account.transaction_list.last
      Result := transaction.account = the_account and
                transaction.type = bc.trans_open and
                transaction.amount = initial_deposit
  	end
  	if Result then comment("Transaction data correct for deposit")
      the_account.deposit(amount)
      transaction := the_account.transaction_list.last
      Result := transaction.account = the_account and
                transaction.type = bc.trans_deposit and
                transaction.amount = amount
    end
  	if Result then comment("Transaction data correct for withdraw")
      the_account.withdraw(amount)
      transaction := the_account.transaction_list.last
      Result := transaction.account = the_account and
                transaction.type = bc.trans_withdraw and
                transaction.amount = amount
    end
  	if Result then comment("Transaction data correct for close")
      the_account.close
      transaction := the_account.transaction_list.last
      Result := transaction.account = the_account and
                transaction.type = bc.trans_close
    end
  	if Result then comment("Transaction data stored correctly") end
  end

end -- class TEST_SAVINGS_ACCOUNT
