indexing
  description: "A bank customer"
  author: "Gunnar Gotshalks"
  date: "2004 February 6"
  version: "1.0"

class CUSTOMER

create make

--========================
feature -- Creation
	
make (a_name : STRING) is
    -- Create a customer and store their name.    
  require  a_name_not_void: a_name /= Void
  do
  	name := a_name
  	create account.make
  ensure
    name_assigned: name = a_name
    empty_account: account.is_empty
  end

--========================
feature -- Access

name :    STRING  -- Customer name
account : LINKED_LIST[SAVINGS_ACCOUNT]

account_exists(acct_number : INTEGER) : BOOLEAN is
  -- Does the customer have an account with number acct_number 
  require true
  do
  	Result := get_account(acct_number) /= void
  ensure Result = (get_account(acct_number) /= void)
  end

balance(acct_number : INTEGER) : REAL is
  -- The balance of the account with number acct_number
  require account_exists(acct_number)
  do
  	Result := get_account(acct_number).balance
  ensure Result = get_account(acct_number).balance
  end
  
--========================
feature -- Element change

create_account : INTEGER is
  -- Create a new account and tell the customer the
  -- account number
  local new_account : SAVINGS_ACCOUNT
  do
    create new_account.make
    account.extend(new_account)
    Result := new_account.number
  ensure
  	one_more_account: account.count = old account.count + 1
  end

open(acct_number: INTEGER; amount : REAL) is
  -- Open the specified account
  require
  	acct_number_exists: account_exists(acct_number)
  	amount_gt_0: amount >= 0
  local
  do
    get_account(acct_number).open(amount)
  end

close(acct_number : INTEGER) is
  -- Close the specified account
  require
  	acct_number_exists: account_exists(acct_number)
  do
  	get_account(acct_number).close
  end

deposit(acct_number : INTEGER; amount : REAL) is
  -- Deposit the amount in the specified account
  require
  	acct_number_exists: account_exists(acct_number)
  	amount_gt_0: amount > 0
  do
  	get_account(acct_number).deposit(amount)
  end

withdraw(acct_number : INTEGER; amount : REAL) is
  -- Withdraw the amount from the specified account
  require
  	acct_number_exists: account_exists(acct_number)
  	amount_gt_0: amount > 0
  do
  	get_account(acct_number).withdraw(amount)
  end

--========================
feature {NONE} -- Support

get_account(acct_number : INTEGER) : SAVINGS_ACCOUNT is
  -- Find the account corresponding to the acct_number
  do
  	from account.start
  	until account.after or account.item.number = acct_number
  	loop
  		account.forth
  	end
  	if not account.after then Result := account.item end
  end

invariant
  name_not_void: name /= Void
  account_list_exists: account /= void
  
end -- class CUSTOMER
