indexing
  description: "Tests for CUSTOMER of bank"
  author: "Gunnar Gotshalks"
  date: "$2004 February 6$"
  revision: "$1.0.0$"
  
class TEST_CUSTOMER
  inherit ES_TEST
  
create make

--========================  
feature {NONE} -- Initialization
make is
    -- Run tests
  do
    all_violation_cases
    all_boolean_cases
  end  

--========================
feature -- Violation

all_violation_cases is
  -- Gather all violation cases together to simplify turning off while
  -- working on the boolean cases
  do
    add_violation_case (agent make_cust_void_name)
    add_violation_case (agent balance_not_exists_acct)
    add_violation_case (agent open_not_exists_acct)
    add_violation_case (agent open_too_small_amt)
    add_violation_case (agent close_not_exists_acct)
    add_violation_case (agent deposit_not_exists_acct)
    add_violation_case (agent deposit_too_small_amt)
    add_violation_case (agent withdraw_not_exists_acct)
    add_violation_case (agent withdraw_too_small_amt)
 end
 
make_cust_void_name is
  local
    the_customer : CUSTOMER
   do
    comment("Precondition customer name void")
    create the_customer.make(void)
  end
  
balance_not_exists_acct is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
    the_balance : REAL
   do
    comment("Precondition balance non existent account")
    create the_customer.make("John Doe")
    the_balance := the_customer.balance(the_account)
  end
  
open_not_exists_acct is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition open non existent account")
    create the_customer.make("John Doe")
    the_customer.open(the_account,100)
  end
  
open_too_small_amt is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition open amount too small")
    create the_customer.make("John Doe")
    the_account := the_customer.create_account
    the_customer.open(the_account, -1)
  end
  
close_not_exists_acct is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition close non existent account")
    create the_customer.make("John Doe")
    the_customer.close(the_account)
  end
  
deposit_not_exists_acct is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition deposit non existent account")
    create the_customer.make("John Doe")
    the_customer.deposit(the_account,100)
  end
  
deposit_too_small_amt is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition open amount too small")
    create the_customer.make("John Doe")
    the_account := the_customer.create_account
    the_customer.deposit(the_account, 0)
  end
  
withdraw_not_exists_acct is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition withdraw non existent account")
    create the_customer.make("John Doe")
    the_customer.withdraw(the_account,100)
  end
  
withdraw_too_small_amt is
  local
    the_account : INTEGER
    the_customer : CUSTOMER
   do
    comment("Precondition withdraw amount too small")
    create the_customer.make("John Doe")
    the_account := the_customer.create_account
    the_customer.withdraw(the_account, 0)
  end
	
--========================
feature -- Boolean  

all_boolean_cases is
  -- Gather all boolean cases as a unit.
  do
    add_boolean_case (agent unique_acct_numbers)
    add_boolean_case (agent proper_balance)
 end

unique_acct_numbers: BOOLEAN is
  local
    account1, account2 : INTEGER
    customer1, customer2: CUSTOMER
  do
    comment("First account numbers are 1 and 2")
    create customer1.make("John Doe")
    account1 := customer1.create_account
    account2 := customer1.create_account
-- The following fails if violations tests are done
--    Result := account1 = 1 and account2 = 2
-- The following tests correctly for unique account numbers,
-- if the accounts are generated in the order 1, 2.
    Result := account1 + 1 = account2
    if Result then comment("Second Customer account number is 3")
      create customer2.make("Anon Ymous")
      account1 := customer2.create_account
      Result := account1 = account2 + 1
    end
    if Result then comment("Account numbers are unique.") end
  end

proper_balance: BOOLEAN is
  local
    account : INTEGER
    the_customer : CUSTOMER
  do
  	create the_customer.make("John Doe")
  	account := the_customer.create_account
    comment("Open with proper balance")
    the_customer.open(account, 100)
    Result := the_customer.balance(account)= 100
    if Result then
      comment("Deposit with proper balance")
      the_customer.deposit(account, 50)
      Result := the_customer.balance(account) = 150
    end
    if Result then
      comment("Withdraw with proper balance")
      the_customer.withdraw(account, 75)
      Result := the_customer.balance(account) = 75
    end
    if Result then comment("Proper balances on all transactions") end
  end
  
end -- class TEST_CUSTOMER
